@@ -7,7 +7,6 @@ use crate::NetPermissions;
 use deno_core::error::bad_resource;
 use deno_core::error::custom_error;
 use deno_core::error::generic_error;
-use deno_core::error::null_opbuf;
 use deno_core::error::type_error;
 use deno_core::error::AnyError;
 use deno_core::op_async;
@@ -36,6 +35,7 @@ use trust_dns_proto::rr::record_type::RecordType;
 use trust_dns_resolver::config::NameServerConfigGroup;
 use trust_dns_resolver::config::ResolverConfig;
 use trust_dns_resolver::config::ResolverOpts;
+use trust_dns_resolver::error::ResolveErrorKind;
 use trust_dns_resolver::system_conf;
 use trust_dns_resolver::AsyncResolver;
 
@@ -167,9 +167,8 @@ pub(crate) struct ReceiveArgs {
 async fn receive_udp(
   state: Rc<RefCell<OpState>>,
   args: ReceiveArgs,
-  zero_copy: Option<ZeroCopyBuf>,
+  zero_copy: ZeroCopyBuf,
 ) -> Result<OpPacket, AnyError> {
-  let zero_copy = zero_copy.ok_or_else(null_opbuf)?;
   let mut zero_copy = zero_copy.clone();
 
   let rid = args.rid;
@@ -197,7 +196,7 @@ async fn receive_udp(
 async fn op_datagram_receive(
   state: Rc<RefCell<OpState>>,
   args: ReceiveArgs,
-  zero_copy: Option<ZeroCopyBuf>,
+  zero_copy: ZeroCopyBuf,
 ) -> Result<OpPacket, AnyError> {
   match args.transport.as_str() {
     "udp" => receive_udp(state, args, zero_copy).await,
@@ -218,12 +217,11 @@ struct SendArgs {
 async fn op_datagram_send<NP>(
   state: Rc<RefCell<OpState>>,
   args: SendArgs,
-  zero_copy: Option<ZeroCopyBuf>,
+  zero_copy: ZeroCopyBuf,
 ) -> Result<usize, AnyError>
 where
   NP: NetPermissions + 'static,
 {
-  let zero_copy = zero_copy.ok_or_else(null_opbuf)?;
   let zero_copy = zero_copy.clone();
 
   match args {
@@ -540,7 +538,7 @@ where
 
 #[derive(Serialize, PartialEq, Debug)]
 #[serde(untagged)]
-enum DnsReturnRecord {
+pub enum DnsReturnRecord {
   A(String),
   Aaaa(String),
   Aname(String),
@@ -585,7 +583,7 @@ pub struct NameServer {
   port: u16,
 }
 
-async fn op_dns_resolve<NP>(
+pub async fn op_dns_resolve<NP>(
   state: Rc<RefCell<OpState>>,
   args: ResolveAddrArgs,
   _: (),
@@ -633,7 +631,19 @@ where
   let results = resolver
     .lookup(query, record_type, Default::default())
     .await
-    .map_err(|e| generic_error(format!("{}", e)))?
+    .map_err(|e| {
+      let message = format!("{}", e);
+      match e.kind() {
+        ResolveErrorKind::NoRecordsFound { .. } => {
+          custom_error("NotFound", message)
+        }
+        ResolveErrorKind::Message("No connections available") => {
+          custom_error("NotConnected", message)
+        }
+        ResolveErrorKind::Timeout => custom_error("TimedOut", message),
+        _ => generic_error(message),
+      }
+    })?
     .iter()
     .filter_map(rdata_to_return_record(record_type))
     .collect();
